 

import java.util.HashMap;

public class NLLambda extends NLTerm {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  NLTerm[] vv,tt;
  NLInfixrel alpha;
  NLFormula phi;
  NLTerm t;
  
  String translation = "";
  boolean isInWords = false;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public NLLambda(NLTerm[] vv, NLInfixrel alpha, NLTerm[] tt,
                  NLFormula phi, NLTerm t) {
    this.vv = vv; this.alpha = alpha; this.tt = tt;
    this.phi = phi; this.t = t;
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  public String xlat(int mode) {
    //First call the xlat methods of the children, and figure out
    //if we are forced to return words.
    //Start with the variables in vv.
    for (int i = 0; i < vv.length; i++) {
      vv[i].xlat(mode); this.isInWords |= vv[i].isInWords();
    }
    //Now treat the prepositional alpha, if it is there.
    if (alpha!=null) {
      boolean prepWords = this.isInWords();
      if (tt==null) {System.err.println("Prep. infix w/o RHS!");} else {
        for (int i = 0; i < tt.length; i++) {
          tt[i].xlat(mode); prepWords |= tt[i].isInWords();
        }
      }
      if (prepWords || mode==NLPhrase.WORDS_MODE) {
        alpha.xlat(NLInfixrel.QUALIFIER_CODE);
      }
      else {alpha.xlat(NLInfixrel.DEFAULT_CODE);}
      prepWords |= alpha.isInWords();
      this.isInWords |= prepWords;
    }
    //Now phi, if it is there.
    if (phi!=null) {
      phi.xlat(mode); this.isInWords |= phi.isInWords();
    }
    //And the term t.
    t.xlat(mode); this.isInWords |= t.isInWords();
    //Finally, write the string.
    String x = "";
    if (this.isInWords || mode==NLPhrase.WORDS_MODE) {
      x = writeWords();
    } else {
      x = writeSymbols();
    }
    this.translation = x;
    return x;
  }

  public boolean isInWords() {
  //Don't call until *after* calling xlat.
    return isInWords;
  }

  public String getString() {return translation;}
  
  String writeSymbols() {
  //If you've settled on writing symbols, call this method to
  //produce the string.
    String s = "";
    s += "(\\lambda ";
    for (int i = 0; i < vv.length; i++) {
      s += (i>0?",":"")+vv[i].getString();
    }
    if (alpha!=null) {
      s += alpha.getString();
      for (int i = 0; i < tt.length; i++) {
        s += (i>0?",":"")+tt[i].getString();
      }
    }
    if (phi!=null) {
      s += " : "+phi.getString();
    }
    s += ")"+t.getString();
    s = Evaluable.ddStrip(s);
    return "$"+s+"$";
  }
  
  String writeWords() {
  //If you've settled on writing words, call this method to
  //produce the string.
    String s = "";
    s += "the function mapping ";
    for (int i = 0; i < vv.length; i++) {
      s += (i>0?", ":"")+vv[i].getString();
    }
    if (alpha!=null) {
      s += alpha.getString();
      for (int i = 0; i < tt.length; i++) {
        s += (i>0?", ":"")+tt[i].getString();
      }
    }
    if (phi!=null) {
      s += " such that "+phi.getString();
    }
    s += ", to "+t.getString();
    return s;
  }


// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 
