
import java.util.HashMap;

public class NLRelation extends NLFormula {
    
// ----------------------------------------------------------------------------
// Constructor
// ----------------------------------------------------------------------------

  public NLRelation(boolean asserted, String s, NLTerm[] tt) {
    this.asserted = asserted; rstr = s; args = tt;
  }

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  boolean asserted;
  String rstr;
  NLTerm[] args;
  String[] argStrings;
  boolean forceWords = false;
  boolean isInWords = false;
  String translation = "";
  
  static final int DEFAULT = 1;
  static final int WORDS = 2;
  static final int POSITED = 4;
  static final int NEGATED = 8;
  static final int SINGULAR = 16;
  static final int PLURAL = 32;
  static final int VERBAL = 64;
  static final int PREPOSITIONAL = 128;

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  boolean getAsserted() {
    return asserted;
  }
  
  String getRstr() {
    return rstr;
  }
  
  NLTerm[] getArgs() {
    return args;
  }
  
  public String xlat(int code) {
    //Impose the asserted condition
    if (asserted) {
      code |= POSITED;
      code &= (~NEGATED);
    } else {
      code |= NEGATED;
      code &= (~POSITED);
    }
    //Proceed.
    String x = "";
    processArgs(code);
    String arity = getArity();
    HashMap map = NLLookup.lookUp(rstr,arity);
    String phrase = getPhrase(code,map);
        //diag:
        if (false) {
          System.out.println(
            (String) map.get("symb") + "\n" +
            (String) map.get("nsym") + "\n" +
            (String) map.get("reln") + "\n" +
            (String) map.get("negn") + "\n" +
            (String) map.get("plur") + "\n" +
            (String) map.get("nplu") + "\n" +
            (String) map.get("prep")
          );
        }
        //
    if (phrase==null) {
      x = makeDefaultPhrase(code);
    } else {
      NLPhraseEvaluator ev = new NLPhraseEvaluator(phrase);
      x = ev.eval(argStrings);
    }
    this.translation = x;
    return x;
  }

  public boolean isInWords() {
  //Should call xlat before this method.
    return isInWords;
  }
  
  public String getString() {
  //Should call xlat before this method.
    return translation;
  }
  
  void processArgs(int code) {
    if (args!=null) {
      argStrings = new String[args.length];
      for (int i = 0; i < args.length; i++) {
        argStrings[i] = args[i].xlat(NLInfixrel.DEFAULT_CODE);
        forceWords |= args[i].isInWords();
      }
    }
  }
  
  String getArity() {
    String a = "0";
    if (args!=null) {
      a = Integer.toString(args.length);
    }
    return a;
  }
  
  String makeDefaultPhrase(int code) {
  //processArgs should already have been called by this time.
  //For use when the symbol could not be found in the dictionary.
  //We construct the default phrase, which is just the symbol followed
  //by its arguments (if any), in the appropriate sort of braces; and
  //possibly preceded by a negation symbol.
    String r = rstr;
    if (args!=null && args.length>0) {
      r = ((code&NEGATED)!=0?"$\\neg$ ":"")+r;
      r += "[";
      for (int i = 0; i < argStrings.length; i++) {
        r += (i==0?"":",")+argStrings[i];
      }
      r += "]";
    }
    return "$"+r+"$";
  }

  
  String getPhrase(int code, HashMap map) {
    String phrase = null;
    //forceWords should be set up by this time.
    if (forceWords) {
      code |= WORDS;
      code &= (~DEFAULT);
    }
    //Now proceed.
    //First determine if there is a symbolic phrase available.
    boolean symbolsAvailable = false;
    if ((code&POSITED)!=0) {
      phrase = (String) map.get(GrammarConstants.SYMB);
      symbolsAvailable = (phrase!=null);
    } else {
      phrase = (String) map.get(GrammarConstants.NSYM);
      symbolsAvailable = (phrase!=null);
    }
    //At this point phrase is already set up, provided we want
    //symbols.
    if ((code&WORDS)!=0 || !symbolsAvailable) {
    //We want words.
      this.isInWords = true;
      if ((code&PREPOSITIONAL)!=0) {
      //We want the prepositional form.
        phrase = (String) map.get(GrammarConstants.PREP);
      } else {
      //We want a verb form.
        if ((code&SINGULAR)!=0) {
        //We want singular.
          if ((code&POSITED)!=0) {
          //Posited.
            phrase = (String) map.get(GrammarConstants.RELN);
          } else {
          //Negated.
            phrase = (String) map.get(GrammarConstants.NEGN);
          }
        } else {
        //We want plural.
          if ((code&POSITED)!=0) {
          //Posited.
            phrase = (String) map.get(GrammarConstants.PLUR);
          } else {
          //Negated.
            phrase = (String) map.get(GrammarConstants.NPLU);
          }
        }
      }
    }
    return phrase;
  }

}










